<?php

/****************************************************************************

	Copyright (C) 2017-2020  Sendatel S.A.

	This file is part of TUIDD PHP SDK.
	TUIDD PHP SDK is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	TUIDD PHP SDK is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with TUIDD PHP SDK.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

require_once('TuiddException.php');

class TuiddClient
{
	private $m_ClientSecret = '';
	private $m_ClientId = '';

	private $m_UrlRedirect = '';
	private $m_UrlDiscovery = '';
	private $m_UrlTuiddContent = '';
	private $m_UrlProveedoraToken = '';
    private $m_UrlUserInfo = '';
	private $m_UrlUserInfoRedirect = '';
	private $m_UrlSendService = '';
	
	private $m_AuthCode = '';
	private $m_AccessTokenData = '';
	private $m_UserInfo = '';
	private $m_PaymentToken = '';

	private $m_LastServiceResult = '';

	private $m_ContainerDiv = '';
	private $m_tuiddSDKPath = '';
	private $m_Scope = '';
	private $m_State= '';
	private $m_Nonce= '';
	private $m_Security = '';	
	private $m_Language = '';
	private $m_HomePageUrl = '';

	private $m_CallbackFunctionFile = '';
	private $m_CallbackFunction = '';
	private $m_StateNonceCallback = '';

	private $m_init_time = '';
	private $__DEBUG__ = '';
	

	/**
	 * TuiddClient's constructor
	 */
	private function __construct()
	{
		$root_dir = dirname( dirname( dirname( dirname(__FILE__) ) ) ) ;
		$configs = parse_ini_file( $root_dir . '/conf/tuidd.conf' ); 
		
		// leo ruta interna para hacer el discovery 
		// Me devuelve json con url simulado por mcg_urls
		$internal_conf = parse_ini_file( "internal_params.conf" );
	
		if ( $configs === FALSE || $internal_conf === FALSE )
		{
			throw new TuiddException( 'Error al leer archivo de configuración' );
		}

		if ( $configs['redirect_uri'] == '' || empty($configs['redirect_uri']))
		{
			$script_pathname = $_SERVER['PHP_SELF'];
			$pattern = '/index.php/i';
			$script_pathname = preg_replace($pattern, '', $script_pathname);
			$this->m_UrlRedirect = $this->url_origin($_SERVER, false, false) . $script_pathname;
		}
		else
		{
			$this->m_UrlRedirect = $configs['redirect_uri'];
		}
		
		if ( $configs['debug'] == 1 || $configs['debug'] == true ||
			 $configs['debug'] == 'true' || $configs['debug'] == '1') 
		{
			$this->__DEBUG__ = 1;
		}
		else
		{
			$this->__DEBUG__ = 0;
		}
		
		if ($this->__DEBUG__)
		{
			syslog(LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: URI Redirect :: " . $this->m_UrlRedirect);	
		}
		
		$this->m_ClientId		 	 = $configs['client_id'];
		$this->m_ClientSecret	 	 = $configs['client_secret'];
		$this->m_PaymentToken        = $configs['payment_token'];
		$this->m_HomePageUrl 		 = $configs['home_page_url'];
		$this->m_UrlUserInfoRedirect = $configs['home_login_url'];
		
		$this->m_Security     = $configs['security_level'];
		$this->m_Scope        = $configs['scope'];
		$this->m_Language     = $configs['default_language'];
		$this->m_ContainerDiv = $configs['container_div'];
		$this->m_tuiddSDKPath = $configs['tuidd_sdk_path'];
		$this->m_UrlDiscovery = $internal_conf['tuidd_discovery'];
		$this->m_UrlTuiddContent = $internal_conf['tuidd_sdk'];
				
		$this->m_CallbackFunction      = $configs['callback_function'];
		$this->m_StateNonceCallback   = $configs['state_nonce_callback_function'];
		$this->m_CallbackFunctionFile = $configs['callback_function_file'];
	}


    public static function blockStart( $name )
    {
        $root_dir = dirname( dirname( dirname( dirname(__FILE__) ) ) ) ;
		$configs = parse_ini_file( $root_dir . '/conf/tuidd.conf' );
        $client_id = $configs['client_id'];
        $debug = $configs['debug'];

        // Esto se hace porque si hay una session iniciada se va a almacenar
        // para iniciarla de nuevo al terminar los procesos del sdk
        if( session_id() != "" )
        {
            // Se cierra la session aactual
            session_write_close();
            $sess = session_id();
            $sessionName = session_name();
        }
        else
        {
            session_start();
            session_write_close();
            $sess = session_id();
            $sessionName = session_name();	

        }

        ini_set("session.use_strict_mode",true);
        ini_set("session.cookie_httponly",true);
    
        session_name($name);
    
        if(!isset($_COOKIE[$name]))
        {
            $hash = hash('sha256', $client_id);
            if ($debug) {
                syslog (LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: session :: 'Se va a setear esta cookie :: " . $name . ' => ' . $hash );
            }
            $_COOKIE[$name] = $hash;
        }
        else
        {
            if ($debug) {
                syslog (LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: session :: 'ya venia una cookie :: " . $_COOKIE[$name] );
            }
        }

        // Esto levanta la session propia del SDK
        session_id($_COOKIE[$name]);
        session_start();
        session_regenerate_id(true);

        // Guarda la cookie generada
        $_COOKIE[$name] = session_id();

        // retrona el session name y session_id originales para que pueda levantarse esta session
        // luego al terminar la ejecución del sdk
        return array( $sess, $sessionName );
    }


    public static function blockEnd( $sess )
    {
        $root_dir = dirname( dirname( dirname( dirname(__FILE__) ) ) ) ;
		$configs = parse_ini_file( $root_dir . '/conf/tuidd.conf' );
        $debug = $configs['debug'];        

        session_write_close();
        if( $sess[0] )
        {
            if ($debug) {
                syslog (LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: session "  . 'esta es la session que estoy tranfiriendo :: ' . $sess[0] );
            }
            session_name( $sess[1] );
            session_id( $sess[0] );
            session_start();
        }
        else
        {
            session_name( $sess[1] );
            if ($debug) {
                syslog (LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: session "  . 'no venia un previo id de session ' . $sess[1] );
            }
        }
    }


	/**
	 * Gets Singleton's instance of TuiddClient 
	 * @return TuiddClient return's instance
	 */
	public static function getSingleton()
	{
        if ( !isset($this) )
        {
        	$root_dir = dirname( dirname( dirname( dirname(__FILE__) ) ) ) ;
			$configs = parse_ini_file( $root_dir . '/conf/tuidd.conf' ); 
        	$clientId = $configs['client_id'];
        }
        else {
			$clientId = $this->m_ClientId;
        }	

        $tuidd_sess = 'tuidd'.$clientId;
		if ( !isset($_SESSION[$tuidd_sess]) || empty($_SESSION[$tuidd_sess]) ) {
			return new self();
		}
		else
    	{
			if( $_SESSION[$tuidd_sess] instanceof TuiddClient ) {
                return $_SESSION[$tuidd_sess];
            } else {
                $ret = new self();
                $ret = unserialize($_SESSION[$tuidd_sess]);
                return $ret;
            } 
    	}
	}


	/**
	 * Serialize object to session
	 */
	private function updateContent() {
		$_SESSION['tuidd'.$this->m_ClientId] = serialize($this);
	}


	/**
	 * Checks if user is logged in
	 * @return Bool True or Redirect to home page
	 */
	public function isLoggedIn( $sess )
	{
		$timeout = false;
		$loggedIn = false;

		if ( empty( $this->m_AccessTokenData ) || empty($this->m_AccessTokenData['id_token']) )
		{
            $this->logout($sess);
			/* echo 'Timeout Session'; */
			/* session_destroy('sdk-session'); */
			/* $_SESSION['tuidd'.$this->m_ClientId] = ''; */
			/* header("Location:" . $this->m_HomePageUrl ); */
		}
		else if ( ! empty( $this->m_AccessTokenData ) && (! empty($this->m_AccessTokenData['id_token'])) )
		{
			$delay = time() - $this->m_init_time;
			$timeout = ( $delay > (int) $this->m_AccessTokenData['expires_in'] );
			if (! $timeout )
			{
				return true;
			}
		}
		if ( $timeout ) 
		{
            $this->logout($sess);
			/* echo 'Timeout Session'; */
			/* session_destroy('sdk-session'); */
			/* $_SESSION['tuidd'.$this->m_ClientId] = ''; */
			/* header("Location:" . $this->m_HomePageUrl ); */
		}
	}


	/**
	 * LOGOUT and destroy session
	 */
	public function logout($sess)
	{
		$home = $this->m_HomePageUrl;
		#session_destroy('sdk-session');
		$_SESSION['tuidd'.$this->m_ClientId] = '';
        $this::blockEnd($sess);
		header("Location:" . $home );
	}


	/**
	 * Request important urls
	 * @param  none
	 * @return string URI for makieng requests 
	 */
	public function discovery2()
	{
		$arrContextOptions=array(
		    "ssl"=>array(
		    "verify_peer"=>false,
		    "verify_peer_name"=>false,
		    ),
		); 

		$mcg_url = $this->m_UrlDiscovery . "/mcg_urls";
		$urls = file_get_contents( $mcg_url, false, stream_context_create($arrContextOptions));

		$urls_json = json_decode($urls, true);

		$request_url = $urls_json['request_url'];
		$this->m_UrlSendService = $urls_json['send_service_url'];
		$this->m_UrlProveedoraToken = $urls_json['service_provider_url'];
		$this->updateContent();

		if ($this->__DEBUG__)
		{
			syslog(LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: DISCOVERY URLS :: " . $urls);
        }
		return $request_url;
	}


	/**
	 * Request fundamental Tuidd urls
	 * @return String Tuidd URI for making requests 
	 */
	public function discovery()
	{
		$mcg_url = $this->m_UrlDiscovery;

		$headers = array();
			$headers[]     = 'Authorization: Basic '.base64_encode( $this->m_ClientId . ':' . $this->m_ClientSecret );
			$headers[]     = 'Content-Type: application/x-www-form-urlencoded';

		$arrParams = array( 'redirect_uri' => $this->m_UrlRedirect );
		$params = http_build_query($arrParams);

		$result = $this->sendCurl($mcg_url, $params, $headers);

		$response_json = json_decode($result['response'], true);
		$urls_json = $response_json['response']['apis']['operatorid']['link'];
        if ($this->__DEBUG__)
		{
			syslog(LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: DISCOVERY URLS :: " . $urls_json);
        }

		$request_url = $urls_json[0]['href'];
		$this->m_UrlProveedoraToken = $urls_json[1]['href'];
		$this->m_UrlUserInfo = $urls_json[2]['href'];
		$this->m_UrlSendService = $urls_json[3]['href'];

		$this->updateContent();

		if ($this->__DEBUG__)
		{
			syslog(LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: DISCOVERY URLS :: " . $urls);
        }

		return $request_url;
	}


	/**
	 * Request for token after getting the authorization code
	 * @param  String Authorization code
	 * @return Bool  True if succeed
     *   TuiddException if not
	 */
	public function getToken( $code )
	{	
		$this->m_AuthCode = $code;

		$url = $this->m_UrlProveedoraToken;

		$headers = array();
        $headers[]     = 'Authorization: Basic '.base64_encode( $this->m_ClientId . ':' . $this->m_ClientSecret );
        $headers[]     = 'Content-Type: application/x-www-form-urlencoded';

		$arrParams = array('grant_type' => 'authorization_code', 'code' => $this->m_AuthCode, 'redirect_uri' => $this->m_UrlRedirect );
		$params = http_build_query($arrParams);
		
		$result = $this->sendCurl($url, $params, $headers);
		$aux = (array)json_decode($result['response']);	

		if ($this->__DEBUG__)
		{
			syslog(LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: GET TOKEN RTA  :: " . $aux);
		}		

		if ( empty($aux)) 
		{
			throw new TuiddException("Error Durante el pedido del Access Token. Respuesta vacia.");
		}
		else if (! empty($aux['error']))
		{
			throw new TuiddException("Error Durante el pedido del Access Token. La descripcion del error es la siguiente: " . $aux['error_descripcion'] );
		}
		else {
			$this->m_AccessTokenData = $aux;
			$this->m_init_time = time();
			return true;
		}
	}

	/**
	 * Request User Ínformation
	 * @return Bool True if succeed
     *   TuiddException if not
	 */
	public function requestUserInfo()
	{
		$url = $this->m_UrlProveedoraToken;

		$headers = array('Content-Type: application/x-www-form-urlencoded', 'Authorization: Bearer ' . $this->m_AccessTokenData['access_token']);
		$arrParams = array('grant_type' => 'authorization_code', 'code' => $this->m_AuthCode, 'redirect_uri' => $this->m_UrlRedirect);

		$params = http_build_query($arrParams);
		$result = $this->sendCurl($url, $params, $headers);
		$aux = (array)json_decode($result['response']);

		if ($this->__DEBUG__)
		{
			syslog(LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: GET USER INFO RTA  :: " . $aux);
		}

		if (empty($aux)) 
		{
			throw new TuiddException("Error Durante el pedido del Access Token. Respuesta vacia");
		}
		else if (! empty($aux['error_description']))
		{
			throw new TuiddException("Error Durante el pedido del UserInfo. La descripcion del error es la siguiente: " . $aux['error_description'] );
		}
		else {
			$this->m_UserInfo = $aux;
			return true;
		}
	}

	/**
     * @brief Exceute Callback Function
	 * Callback Function for sending information to page. This function can be enabled by SDK configuration.
	 * If not it's only redirect to xxx parameter set in config.
	 */
	public function cfReceive() 
	{
		$user_info = $this->m_UserInfo;
		$user_info['client_info']  = base64_encode( $this->m_ClientId . ':' . $this->m_ClientSecret );
		$user_info['access_token']  = $this->m_AccessTokenData['access_token'];
		$user_info['id_token']  = $this->m_AccessTokenData['id_token'];

		$this->m_UserInfo = $user_info;
		$this->updateContent();

		if ( !empty($this->m_CallbackFunctionFile) && $this->m_CallbackFunctionFile != '' && !empty($this->m_CallbackFunction) && $this->m_CallbackFunction != '')
		{
			require_once( $this->m_CallbackFunctionFile );

			$notification = array();
			$notification['service_type'] = 'authentication';
			$notification['notification_data'] = $this->getUserInfo();

			call_user_func( $this->m_CallbackFunction, $notification );
		}
			
		// Se hace redirect
		header('refresh:1; ' . $this->m_UrlUserInfoRedirect );
	}

	
	/**
	 * Set Nonce atribute. 
	 * @param String nonce
	 */
	public function setNonce ( $nonce )
	{
		$this->m_Nonce = $nonce;
		$this->updateContent();
	}	


	/**
	 * Set State atribute. 
	 * @param String state
	 */
	public function setState ( $state )
	{
		$this->m_State = $state;
		$this->updateContent();
	}


	/**
	 * Function for consuming a service. 
	 * @param  array Service information to be consumed
	 * @return array Response of service
	 */
	public function consumeService ( $service )
	{
		$headers = array();
			$headers[]     = 'Authorization: Bearer '.base64_encode( $this->m_ClientId . ':' . $this->m_ClientSecret );
			$headers[]     = 'Content-Type: Application/json';

		$url = $this->m_UrlSendService;

		// JSON_FORCE_OBJECT, para Array multidimensionales
		$params = json_encode($service, JSON_FORCE_OBJECT);
		
		$result = $this->sendCurl($url, $params, $headers);

		if ($this->__DEBUG__)
		{
			syslog(LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: SERVICIO A CONSUMIR :: " . $params);
			syslog(LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: Respuesta de Servicio :: " . $result['response']);
		}

        $r = $result['response'];
        if (strpos($r, "{" ) > 1)
        {
            $r = substr($r, strpos($r, "{" )-1, strlen($r));
        }
        $response = json_decode( $r, true);

		$st = 1;
		$statusMessage;

		if ($result['status'] == 200)
		{
			if ($response['transaction_id'] && $response['seq_num'] )
			{
				$statusMessage = 'OK';
				$rta = array( 'transaction_id' => $response['transaction_id'],
					'seq_num' => $response['seq_num'],
                );
                $st = 0;
                if ($response['url_redirect'])
                {
                    $rta['url_redirect'] = $response['url_redirect'];
                }
            }
            else
            {                 
                $rta = null; //$response['error_data'];
                $statusMessage = 'Error. Respuesta Incompleta. Falta ';

                if (! $response['transaction_id'])
                {
                    $statusMessage = $statusMessage . 'el parametro transaction_id.';  				
                }
                else if (! $response['seq_num'])
                {
                    $statusMessage = $statusMessage . 'el parametro seq_num.';  				
                }
            }
		}
		else if ($result['status'] == 400)
		{
		    $statusMessage = 'BAD REQUEST';
            if ($response['status']['error_data']['error_description']) {
                $statusMessage = "Error: " . $response['status']['error_data']['error'] ." - ". $response['status']['error_data']['error_description'];
            }
		}
		else if ($result['status'] == 500)
		{
		    $statusMessage = 'INTERNAL SERVER ERROR';
            if ($response['status']['error_data']['error_description']) {
                $statusMessage = "Error: " . $response['status']['error_data']['error'] ." - ". $response['status']['error_data']['error_description'];
            }
		}
		else if ($result['status'] == 401)
		{
		    $statusMessage = 'UNAUTHORIZED';
            if ($response['status']['error_data']['error_description']) {
                $statusMessage = "Error: " . $response['status']['error_data']['error'] ." - ". $response['status']['error_data']['error_description'];
            }
		}

		$serviceResponse = array(
            'result' => $st, 
			'status' => array('description' => $statusMessage,
                              'code' => $result['status']), 
			'response' => $rta, 	
		);

		return $serviceResponse;
	}


	/**
	 * Get Survey service TOKEN. 
	 * @param  array Service information to be consumed
	 * @return array Status
	 */
	public function generateStateAndNonce()
    {
        // Se llama a la funcion de callback establecida en archivo de config
        // para generar state y nonce
        if ( $this->getCallbackFunctionFile() != '' && $this->getStateNonceCallback() != '' )
        {
            require_once( $this->getCallbackFunctionFile() );
            call_user_func( $this->getStateNonceCallback() );
        }
	}

    /**
	 * Genera un state que contiene el state del sitio y un state interno del
	 * sdk.
	 * 
	 * @param  string $clientState el state que notifico el sitio.
	 * @param  string $sdkState    state con informaicon util para el sdk.
	 * @return string              json con los states en base64_url.
	 */
	public function generateSdkState($clientState, $sdkState='')
	{
		$jsonState = json_encode([
			'client_state' => $clientState,
			'sdk_state' => $sdkState
			]);
		if ($this->__DEBUG__)
		{
			syslog(LOG_INFO, "TUIDD SDK :: ". __METHOD__ ." :: state: $jsonState");
		}

		return self::base64urlEncode($jsonState);
	}

	/**
	 * deserializa el rawState devolciendo un array asociativo con los campos:
	 * [ 'client_state' => $clientState, 'sdk_state' => $sdkState ]
	 * 
	 * @param  String $rawState json con los states en base64_url.
	 * @return array            Array con 2 campos client_state y action_state.
	 */
	public function readSdkState($rawState)
	{
		$jsonState = self::base64urlDecode($rawState);

		if ($this->__DEBUG__)
		{
			syslog(LOG_INFO, "TUIDD SDK :: ". __METHOD__ ." :: state: $jsonState");
		}
		$ret = json_decode($jsonState, TRUE);
		if ($this->__DEBUG__)
		{
			syslog(LOG_INFO, "TUIDD SDK :: ". __METHOD__ ." :: return: ".print_r($ret,TRUE));
		}
		return $ret;
	}

	/**
	 * If Discovery URLS had been set
	 * @return Bool
	 */
	public function hasDiscoveryUrls() {
		return ( isset($this->m_UrlProveedoraToken) && $this->m_UrlProveedoraToken != '');
	}


	/**
	 * Getter: Access TOKEN. 
	 * @return String Tuidd Access Token
	 */
	public function getAccessToken() {
		return $this->m_AccessTokenData['access_token'];
	}

	/**
	 * Getter: Survey service TOKEN. 
	 * @return String Survey Service Token
	 */
	public function getSurveyServiceToken() {
		return $this->m_AccessTokenData['tuidd.com.survey_token'];
	}


	/**
	 * Getter: Survey service TOKEN. 
	 * @return String Advertising Service Token
	 */
	public function getAdvertisingServiceToken() {
		return $this->m_AccessTokenData['tuidd.com.advertising_token'];
	}


	/**
	 * Getter: Survey service TOKEN. 
	 * @return String Notification Service Token
	 */
	public function getNotificationServiceToken() {
		return $this->m_AccessTokenData['tuidd.com.notification_token'];
	}


	/**
	 * Getter: Client ID
	 * @return String
	 */
	public function getClientID() {
		return $this->m_ClientId;
	}
	

	/**
	 * Getter: Scope ID
	 * @return String
	 */
	public function getScope()
    {
		return $this->m_Scope;
	}
	

	/**
	 * Getter: Security level (ACR values)
	 * @return String
	 */
	public function getSecurityLevel()
    {
		return $this->m_Security;
	}
	

	/**
	 * Getter: Redirect URI
	 * @return String
	 */
	public function getRedirectURI()
    {
		return $this->m_UrlRedirect;
	}
	

	/**
	 * Getter: SDK Language
	 * @return String
	 */
	public function getLanguage()
    {
		return $this->m_Language;
	}
	

	/**
	 * Getter: SDK container div
	 * @return String
	 */
	public function getContainerDiv()
    {
		return $this->m_ContainerDiv;
	}
	

	/**
	 * Getter: SDK Path
	 * @return String
	 */
	public function getTuiddSDKPath()
    {
		return $this->m_tuiddSDKPath;
	}


	/**
	 * Getter: SDK Home page URL
	 * @return String
	 */
	public function getHomePageURL()
    {
		return $this->m_HomePageUrl;
	}

	
    /**
	 * Getter: SDK Login Home page URL
	 * @return String
	 */
	public function getLoginHomeURL()
    {
		return $this->m_UrlUserInfoRedirect;
	}


	/**
	 * Getter: Discovery URL
	 * @return String
	 */
	public function getDiscoveryUrl()
    {
		return $this->m_UrlDiscovery;
	}


	/**
	 * Getter: URL for requesting tuidd button files
	 * @return String
	 */
	public function getTuiddContentURL()
    {
		return $this->m_UrlTuiddContent;
	}


	/**
	 * Getter: Logged in user Information
	 * @return array  userInfo
	 */
	public function getUserInfo()
    {
		return $this->m_UserInfo;
	}


	/**
	 * Getter: File to place callback functions. 
	 * @return String
	 */	
	public function getCallbackFunctionFile()
	{
		return $this->m_CallbackFunctionFile;
	}


	/**
	 * Getter: function name to be call to generate State and Nonce
	 * @return String Function name
	 */	
	public function getStateNonceCallback()
	{
		return $this->m_StateNonceCallback;
	}


	/**
	 * Getter: state
	 * @return state
	 */	
	public function getState()
	{
		return $this->m_State;
	}


	/**
	 * Getter: nonce
	 * @return nonce
	 */	
	public function getNonce()
	{
		return $this->m_Nonce;
	}


	/**
	 * consulta si esta habilitado el debug
	 * @return boolean isDebugging
	 */	
	public function isDebugging()
	{
		return $this->__DEBUG__;
	}


	/**
	 * Validates id_token
	 * @return Bool True if valid
	 */
	public function validateLogin() 
	{
		$parent = dirname( dirname( dirname( __FILE__ ) ) );
		require_once( $parent . '/jwtManager/JWT.php' ); 

		$id_token = $this->m_AccessTokenData['id_token'];
		$cs = $this->m_ClientSecret;
		
		try 
		{
			$decoded = JWT::decode($id_token, $cs, array('HS256'));
			$decoded = (array) $decoded;

			if (!$decoded)
			{
				// Mensaje de error ?
				// que se hace  
			}

			return  ($this->m_Nonce === $decoded->nonce);
		} 
		catch(Exception $e) 
		{
			echo 'Message: ' .$e->getMessage();
		}		
	}


	/**
	 * Make POST requests 
	 * @params url enpoint where to make the request
	 * @params params argumentes to be passed to url
	 * @params headers http headers for the url
	 * @return array Response
	 */
    private	function sendCurl($url,$params, $headers)
	{
		$params = urldecode($params);
		$ch = curl_init();

		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HTTPGET, FALSE);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $params);

		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false);
			   
		curl_setopt($ch, CURLOPT_HEADER, true);
		curl_setopt($ch, CURLOPT_HTTPHEADER,  $headers);

		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);

		$response = curl_exec($ch);
		$header_size     = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
		$responseBody    = substr($response, $header_size);
		$http_status     = curl_getinfo($ch, CURLINFO_HTTP_CODE);

		if ( curl_error($ch) )
		{
			curl_close($ch);
			throw new Exception("Error Durante el envio de Curl");
		}
		curl_close($ch);
		
		return array(
            'status' => $http_status,
            'response' => $responseBody
        );
	}	


	/**
	 * Compute URL of site
	 * @params s server variables 
	 * @params use_forwarder_host 
	 * @params force_https Optional  
	 * @return array Site URL
	 */
	private function url_origin($s, $use_forwarded_host=false, $force_https=false)
	{
		if (!$force_https)
		{
			$ssl = ( ! empty($s['HTTPS']) && $s['HTTPS'] == 'on' ) ? true:false;
			$sp = strtolower( $s['SERVER_PROTOCOL'] );
			$protocol = substr( $sp, 0, strpos( $sp, '/'  )) . ( ( $ssl ) ? 's' : '' );
		}
		else
		{
			$protocol = 'https';
		}

		$port = $s['SERVER_PORT'];
		$port = ( ( ! $ssl && $port == '80' ) || ( $ssl && $port=='443' ) ) ? '' : ':' . $port;

		$host = ( $use_forwarded_host && isset( $s['HTTP_X_FORWARDED_HOST'] ) ) ? $s['HTTP_X_FORWARDED_HOST'] : ( isset( $s['HTTP_HOST'] ) ? $s['HTTP_HOST'] : null );
		$host = isset( $host ) ? $host : $s['SERVER_NAME'] . $port;

		return $protocol . '://' . $host;
	}


	public static function base64urlEncode($str)
	{
		return rtrim( strtr( base64_encode( $str ), '+/', '-_'), '=');
	}


	public static function base64urlDecode($str)
	{
		return base64_decode( strtr( $str, '-_', '+/') . str_repeat('=', 3 - ( 3 + strlen( $str )) % 4 ));
	}
}

?>
