<?php

/****************************************************************************

	tuidd's PHP SDK version 2.0.0

	Copyright (C) 2017-2020  Sendatel S.A.

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

/****************************************************************************/

require_once('TuiddException.php');
require_once('TuiddClient.php');

class TuiddServicesClient
{
    private $client_id;

	private function __construct()
	{
        $root_dir = dirname( dirname( dirname( dirname(__FILE__) ) ) ) ;
        $configs = parse_ini_file( $root_dir . '/conf/tuidd.conf' ); 

		if ( $configs === FALSE  )
		{
			throw new TuiddException( 'Error al leer archivo de configuración' );
		}
        $this->clientId = $configs['client_id'];
	}

	/**
	 * Gets Singleton's instance of TuiddClient 
	 * @param  none
	 * @return [type]              [description]
	 */
	public static function getSingleton()
	{
		if ( !isset($this) )
        {
        	$root_dir = dirname( dirname( dirname( dirname(__FILE__) ) ) ) ;
			$configs = parse_ini_file( $root_dir . '/conf/tuidd.conf' ); 
            if ( $configs === FALSE  ) {
                throw new TuiddException( 'Error al leer archivo de configuración' );
            }
        	$clientId = $configs['client_id'];
        }
        else {
			$clientId = $this->client_id;
        }	

		if ( !isset($_SESSION['tuiddsc'.$clientId]) || empty($_SESSION['tuiddsc'.$clientId]) ) {
			return new self();
		}
		else
    	{
            if( $_SESSION['tuiddsc'.$clientId] instanceof TuiddServicesClient ) {
                return $_SESSION['tuiddsc'.$clientId];
            } else {
                $ret = new self();
                $ret = unserialize($_SESSION['tuiddsc'.$clientId]);
                return $ret;
            } 
    	}
	}


	/**
	 * Envía una publicidad con la información recibida en los parámetros. 
	 * 
	 * @param  string service_id [ID de servicio]
	 * @param  array  authInfo [description] 
	 * @param  string title [titulo de la publicidad]
	 * @param  string message [mensaje de la publicidad]
	 * @param  string link [link de la publicidad]
	 * @param  string image_link [link de la imagen publicidad]
	 * @param  string redirect_link [link de la redirect luego de recibir publicidad]
	 * @param  array  extrParams [description]
	 * @return array
	 *  int $status Estado de envío de publicidad
	 *      200: OK
	 *      400: BAD REQUEST 
	 *      401: UNAUTHORIZED
     *      500: INTERNAL SERVER ERROR
	 *  int $transaction_id ID de transacción de servicio
	 */
	public function sendAdvertisement($service_id, $authInfo, $title=NULL, $message=NULL, $link=NULL, $image_link=NULL, $redirect_link=NULL, $source=null, $extraParams=NULL )
	{
		TuiddClient::getSingleton()->generateStateAndNonce();
        $tuidd = TuiddClient::getSingleton();
        $state = $tuidd->getState();
        $serviceInfo = array( );

        if ( !$authInfo ) {
            return json_encode(array('error' => 'Parametro AuthInfo vacio'));
        }

        if ( $service_id ) {
            $serviceInfo['service_id'] = $service_id; 

            if ( $title )
            {
                $serviceInfo['title'] = $title;
            }
            if ( $message )
            {
                $serviceInfo['message'] = $message;
            }
            if ( $link )
            {
                $serviceInfo['link'] = $link;
            }
            if ( $image_link )
            {
                $serviceInfo['image_link'] = $image_link;	
            }	

            if ($redirect_link)
            {
                $serviceInfo['redirect_link'] = $redirect_link;
            }
        }
        /* else { */
        /*     return json_encode(array('error' => 'Service ID vacio')); */
        /* } */

 		if ($source)
        {
            if ($service_id) {
                if ($source != 'service_id')
                    return json_encode(array('error' => 'El campo source debe ser "service_id" cuando se envia Service ID'));
                $serviceInfo['source'] = 'service_id';
            }
            else {
                $serviceInfo['source'] = $source;
            }
        }
        else {
            $serviceInfo['source'] = 'any';
        }

		$display = 'url_redirect';
		if ($extraParams['display']) 
		{
			$display = $extraParams['display'];
		}
		
		$service = array(
			'service_token' => $authInfo['service_token'],
			'seq_num' => mt_rand(1,20),
			'state' => $state,
			'login_hint' => $authInfo['login_hint'],
			'service_type' => 'advertising',
			'display' => $display,
			'service_info' => $serviceInfo
		);

		$result = $tuidd->consumeService( $service );
		
		return $result;
	}

    /**
	 * Consume un servicio de pago . 
	 * 
	 * @param  float  price [titulo de la pago]
	 * @param  string detail [description]
	 * @param  string concept [description]
	 * @param  string currency [description]
	 * @param  array  authInfo [description]
	 * @param  array  extraParams [description]
	 * @return array
	 *  int $status Estado de envío de pago
	 *      200: OK
	 *      400: BAD REQUEST 
	 *      401: UNAUTHORIZED
     *      500: INTERNAL SERVER ERROR
	 *  int $transaction_id ID de transacción de servicio
	 */
	public function sendPayment ( $price, $detail, $concept, $currency, $authInfo, $extraParams=NULL )
	{
		TuiddClient::getSingleton()->generateStateAndNonce();
        $tuidd = TuiddClient::getSingleton();
        $state = $tuidd->getState();

        if ( !$authInfo ) {
            return json_encode(array('error' => 'Parametro AuthInfo vacio'));
        }

        if ( $price && $detail && $concept && $currency )
        {
            $serviceInfo = array( 
                'price' => $price,
                'detail' => $detail,
                'concept' => $concept,
                'currency' => $currency,
            );
        }
        else {
            return json_encode(array('error' => 'Faltante de parametros de servicio de pago'));
        }

		$display = 'url_redirect';
		if ($extraParams['display']) 
		{
			$display = $extraParams['display'];
		}

		$service = array(
			'service_token' => $authInfo['service_token'],
			'seq_num' => mt_rand(1,20),
			'state' => $state,
			'login_hint' => $authInfo['login_hint'],
			'service_type' => 'payment',
			'notification_uri' => $extraParams['notification_uri'],
			'display' => $display,
			'service_info' => $serviceInfo
		);

		$result = $tuidd->consumeService( $service);

		return $result;
	}


	/**
	 * Consume un servicio de notificacion. 
	 * 
	 * @param  string service_id [ID de servicio]
	 * @param  array  authInfo [description]
	 * @param  string title [titulo de la notificacion]
	 * @param  string message [mensaje de la notificacion]
	 * @param  string link [link de la notificacion]
	 * @param  array  extraParams [description]
	 * @return array
	 *  int $status Estado de envío de notificacion
	 *      200: OK
	 *      400: BAD REQUEST 
	 *      401: UNAUTHORIZED
     *      500: INTERNAL SERVER ERROR
	 *  int $transaction_id ID de transacción de servicio
	 */
	public function sendNotification ( $service_id,  $authInfo, $title=NULL, $message=NULL, $link=NULL, $extraParams = NULL)
	{
		TuiddClient::getSingleton()->generateStateAndNonce();
        $tuidd = TuiddClient::getSingleton();
        $state = $tuidd->getState();

        if ( $service_id ) {
            $serviceInfo = array( 
                'service_id' => $service_id
            );
        }
        else {
            return json_encode(array('error' => 'Parametro Service ID vacio'));
        }

        if ( !$authInfo ) {
            return json_encode(array('error' => 'Parametro AuthInfo vacio'));
        }

        if ( $title )
        {
            $serviceInfo['title'] = $title;
        }
        if ( $message )
        {
            $serviceInfo['message'] = $message;
        }
        if ( $link )
        {
            $serviceInfo['link'] = $link;
        }
        

		$display = 'url_redirect';
		if ($extraParams['display']) 
		{
			$display = $extraParams['display'];
		}

		$service = array(
			'service_token' => $authInfo['service_token'],
			'seq_num' => mt_rand(1,20),
			'state' => $state,
			'login_hint' => $authInfo['login_hint'],
			'service_type' => 'notification',
			'notification_uri' => $extraParams['notification_uri'],
			'display' => $display,
			'service_info' => $serviceInfo
		);

		$result = $tuidd->consumeService( $service );

		return $result;
	}


	/**
	 * Consume un servicio de encuesta mediante el id del servicio. 
	 * 
	 * @param  string serviceID [titulo de la encuesta]
	 * @param  array  authInfo [description]
	 * @param  array  extraParams [description]
	 * @return array
	 *  int $status Estado de envío de encuesta
	 *      200: OK
	 *      400: BAD REQUEST 
	 *      401: UNAUTHORIZED
     *      500: INTERNAL SERVER ERROR
	 *  int $transaction_id ID de transacción de servicio
	 */
	public function sendSurvey ( $service_id, $authInfo, $extraParams = NULL)
	{
		TuiddClient::getSingleton()->generateStateAndNonce();
        $tuidd = TuiddClient::getSingleton();
        $state = $tuidd->getState();

        if ( !$authInfo ) {
            return json_encode(array('error' => 'Parametro AuthInfo vacio'));
        }

		$display = 'url_redirect';
		if ($extraParams['display'])
		{
			$display = $extraParams['display'];
		}

		$service = array(
			'service_token' => $authInfo['service_token'],
			'seq_num' => mt_rand(0,1000),
			'state' => $state,
			'login_hint' => $authInfo['login_hint'],
			'service_type' => 'survey',
			'display' => $display,
			'service_info' => array(
					'service_id' => $service_id
				)
		);

		$result = $tuidd->consumeService( $service );

		return $result;
	}


	/**
	 * Envía una publicidad con la información recibida en los parámetros. 
	 * 
	 * @param  string serviceID Id de servicio
	 * @param  2D-Array authInfos Informacion de autorizacion por cada servicio
	 * @param  array  extraParams Parametros adicionales a incluir en c/ servicio
	 * @return 2D-Array respuestas de cada servicio
	 *  int $status Estado de envío de publicidad
	 *      200: OK
	 *      400: BAD REQUEST 
	 *      401: UNAUTHORIZED
     *      500: INTERNAL SERVER ERROR
	 *  int $transaction_id ID de transacción de servicio
	 */
	public function sendMultipleAdvertisements($service_id, $authInfos, $extraParams=NULL )
	{
        $seq_num = mt_rand(0,1000);
        $results = array();

        foreach ($authInfos as $index => $authInfo)
        {

            TuiddClient::getSingleton()->generateStateAndNonce();
            $tuidd = TuiddClient::getSingleton();
            $state = $tuidd->getState();

            if ( $service_id ) {
                $serviceInfo = array( 
                    'service_id' => $service_id
                );
            }
            else {
                return json_encode(array('error' => 'Service ID vacio'));
            }

            if ( !$authInfo ) {
                return json_encode(array('error' => 'Parametro AuthInfo vacio'));
            }

            /* if ($redirect_link) */
            /*     { */
            /*         $serviceInfo['redirect_link'] = $redirect_link; */
            /*     } */

            $display = 'url_redirect';
            if ($extraParams['display']) 
                {
                    $display = $extraParams['display'];
                }
		
            $service = array(
                'service_token' => $authInfo['service_token'],
                'seq_num' => $seq_num+$index,
                'state' => $state,
                'login_hint' => $authInfo['login_hint'],
                'service_type' => 'advertising',
                'display' => $display,
                'service_info' => $serviceInfo
            );

            $result = $tuidd->consumeService( $service );
            array_push($results, $result);
        }
		
		return $results;
	}


	/**
	 * Consume multiples servicios de encuestas mediante. 
	 * 
	 * @param  string serviceID Id de servicio
	 * @param  2D-Array authInfos Informacion de autorizacion por cada servicio
	 * @param  array  extraParams Parametros adicionales a incluir en c/ servicio
	 * @return 2D-Array respuestas de cada servicio
	 *  int $status Estado de envío de encuesta
	 *      200: OK
	 *      400: BAD REQUEST 
	 *      401: UNAUTHORIZED
     *      500: INTERNAL SERVER ERROR
	 *  int $transaction_id ID de transacción de servicio
	 */
	public function sendMultipleSurveys ( $service_id, $authInfos, $extraParams = NULL)
	{
        $seq_num = mt_rand(0,1000);
        $results = array();

        foreach ($authInfos as $index => $authInfo)
        {
            TuiddClient::getSingleton()->generateStateAndNonce();
            $tuidd = TuiddClient::getSingleton();
            $state = $tuidd->getState();

            if ( !$authInfo ) {
                return json_encode(array('error' => 'Parametro AuthInfo vacio'));
            }

            $display = 'url_redirect';
            if ($extraParams['display'])
                {
                    $display = $extraParams['display'];
                }

            $service = array(
                'service_token' => $authInfo['service_token'],
                'seq_num' => $seq_num+$index,
                'state' => $state,
                'login_hint' => $authInfo['login_hint'],
                'service_type' => 'survey',
                'display' => $display,
                'service_info' => array(
					'service_id' => $service_id
				)
            );

            $result = $tuidd->consumeService( $service );
            array_push($results, $result);
        }

		return $results;
	}


	/**
	 * Consume un servicio de cupon mediante la informacion enviada por parametro
	 * 
	 * @param  string service_id [ID de servicio]
	 * @param  array  authInfo [description]
	 * @param  string title [titulo del cupon]
	 * @param  string message [mensaje del cupon]
	 * @param  string coupon_type [tipo de cupon ]
	 * @param  string link [link del cupon]
	 * @param  string image_link [link de la imagen de cupon]
	 * @param  array  extraParams [description]
	 * @return array
	 *  int $status Estado de envío de publicidad
	 *      200: OK
	 *      400: BAD REQUEST 
	 *      401: UNAUTHORIZED
     *      500: INTERNAL SERVER ERROR
	 *  int $transaction_id ID de transacción de servicio
	 */
	public function sendCoupon ( $service_id, $authInfo, $title=NULL, $message=NULL, $coupon_type=NULL, $link=NULL, $image_link=NULL, $extraParams = NULL )
	{
		TuiddClient::getSingleton()->generateStateAndNonce();
        $tuidd = TuiddClient::getSingleton();
        $state = $tuidd->getState();

        if ( $service_id ) {
            $serviceInfo = array( 
                'service_id' => $service_id
            );
        }
        else {
            return json_encode(array('error' => 'Parametro Service ID vacio'));
        }

        if ( !$authInfo ) {
            return json_encode(array('error' => 'Parametro AuthInfo vacio'));
        }

        if ( $title )
        {
            $serviceInfo['title'] = $title;
        }
        if ( $message )
        {
            $serviceInfo['message'] = $message;
        }
        if ( $link )
        {
            $serviceInfo['link'] = $link;
        }
        if ( $image_link )
        {
            $serviceInfo['image_link'] = $image_link;	
        }
        if ( $coupon_type )
        {
            $serviceInfo['coupon_type'] = $coupon_type;
        }

		$display = 'url_redirect';
		if ($extraParams['display']) 
		{
			$display = $extraParams['display'];
		}

		$service = array(
			'service_token' => $authInfo['service_token'],
			'seq_num' => mt_rand(1,20),
			'state' => $state,
			'login_hint' => $authInfo['login_hint'],
			'service_type' => 'coupon',
			'display' => $display,
			'service_info' => $serviceInfo
		);
		
		$result = $tuidd->consumeService( $service );
		return $result;
	}
}

?>
