<?php

/****************************************************************************

    Copyright (C) 2017-2020  Sendatel S.A.

    This file is part of TUIDD PHP SDK.
    TUIDD PHP SDK is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TUIDD PHP SDK is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TUIDD PHP SDK.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

require_once('client/TuiddException.php');
require_once('client/TuiddClient.php');


function tuidd_render_login_button()
{
    $parent_dir = dirname( dirname( dirname(__FILE__) ) ) ;

    $configs = parse_ini_file( $parent_dir . '/conf/tuidd.conf' );
    $internal_conf = parse_ini_file( 'client/internal_params.conf' );

    echo '<script type="text/javascript" src="'. $configs['tuidd_sdk_path'] . 'pub/js/jquery/jquery-1.8.3.js"></script>';
    echo '<script type="text/javascript"> var own_jQuery = $.noConflict(true); </script>';

    if ($configs === FALSE) {
        throw new Exception('El archivo de configuracion /conf/tuidd.conf no pudo ser leido ');
    }
    if ($internal_conf === FALSE) {
        throw new Exception('El archivo de configuracion ./client/internal_params.conf no pudo ser leido');
    }
    
    if ( $configs['debug'] == 1 || $configs['debug'] == true ||
             $configs['debug'] == 'true' || $configs['debug'] == '1') 
    {
        $DEBUG = 1;
    }
    else
    {
        $DEBUG = 0;
    }
    
    if ($DEBUG)
    {
        syslog (LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: CONFIGURACION "  . print_r( json_encode($configs), true) );
        syslog (LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: CURL Version :: " . print_r( _hasCurl(), true) );
    }   

    $sess_ = TuiddClient::blockStart('sdk-session');

    $session_lang = 'lang-'.$configs['client_id'];
    $session_locale = 'locale-'.$configs['client_id'];

    if ( !isset($_SESSION[$session_lang])  || !isset($_SESSION[$session_locale]))
    {
        if ($configs['default_language'] == 'en') {
                $file_content = curl_file_get_contents( $internal_conf['tuidd_sdk'] . '/language/tuidd-en_US');
                if ($file_content === FALSE) {
                    throw new Exception('No se pudo leer el archivo de traducciones: '.$internal_conf['tuidd_sdk'] . '/language/tuidd-en_US');
                }
                $lang = parse_ini_string( $file_content );
                $locale = 'en_US';      
        } elseif ($configs['default_language'] == 'es') {
                $file_content = curl_file_get_contents( $internal_conf['tuidd_sdk'] . '/language/tuidd-es_AR');
                if ($file_content === FALSE) {
                    throw new Exception('No se pudo leer el archivo de traducciones: '.$internal_conf['tuidd_sdk'] . '/language/tuidd-es_AR');
                }
                $lang = parse_ini_string( $file_content );
                $locale = 'es_AR';
        } elseif ($configs['default_language'] == 'pt') {
            $file_content = file_get_contents( $internal_conf['tuidd_sdk'] . '/language/tuidd-pt_BR');
                if ($file_content === FALSE) {
                    throw new Exception('No se pudo leer el archivo de traducciones: '.$internal_conf['tuidd_sdk'] . '/language/tuidd-pt_BR');
                }
                $lang = parse_ini_string( $file_content );
                $locale = 'pt_BR';
        } else {
            $file_content = file_get_contents( $internal_conf['tuidd_sdk'] . '/language/tuidd-en_US');
            if ($file_content === FALSE) {
                throw new Exception('No se pudo leer el archivo de traducciones: '.$internal_conf['tuidd_sdk'] . '/language/tuidd-en_US');
            }
            $lang = parse_ini_string( $file_content );
            $locale = 'en_US';
        }	
        $_SESSION[$session_lang] = $lang;
        $_SESSION[$session_locale] = $locale;
    }
    else
    {
        $lang = $_SESSION[$session_lang]; 
        $locale = $_SESSION[$session_locale];
    }
    if ($DEBUG)
    {
        syslog (LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: SESSION :: "  . print_r( $_SESSION, true) );
        syslog (LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: RESTAURO SESS ::"  . print_r( json_encode($sess_), true) );
    }

    TuiddClient::blockEnd($sess_);


    // CSS files
    echo '<link rel="stylesheet" href="' . $internal_conf['tuidd_sdk'] . 'css/tuidd.min.css" type="text/css">';
    echo '<link rel="stylesheet" href="' . $configs['tuidd_sdk_path'] . 'pub/css/sdk.css" type="text/css">';
    echo '<link rel="stylesheet" href="' . $internal_conf['tuidd_sdk'] . 'css/carousel.min.css" type="text/css">';
    echo '<link rel="stylesheet" href="' . $configs['tuidd_sdk_path'] . 'pub/js/intl-tel-input/build/css/intlTelInput.css" type="text/css">';

    // JAVASCRIPTS files
    echo '<script type="text/javascript" src="'. $configs['tuidd_sdk_path'] . 'pub/js/intl-tel-input/build/js/intlTelInput.js"></script>';
    echo '<script type="text/javascript" src="'. $configs['tuidd_sdk_path'] . 'pub/js/jquery-cookie/jquery.cookie.js"></script>';
    echo '<script type="text/javascript" src="' . $internal_conf['tuidd_sdk'] . '/js/jsButtonWidget.js"></script>';
    echo '<script type="text/javascript" src="' . $internal_conf['tuidd_sdk'] . '/js/carousel.min.js"></script>';

    // Languajes' vars to javascript
    echo '<script type="text/javascript">var lang = '. html_entity_decode(json_encode($lang))  . ';</script>';
    // SDK_path to javascript
    echo '<script type="text/javascript">var sdk_path = "'. $configs['tuidd_sdk_path']  .'";</script>';

    $properties = array(
        'type' => 'php',
        'plugin_url' => $configs['tuidd_sdk_path'],
        'callback' => '',
        'lang' => $lang,
        'tuidd_users_url' => $internal_conf['tuidd_users_url'],
        'debug' => $DEBUG
    );
    echo '<script type="text/javascript">var sdk_properties = '.html_entity_decode(json_encode($properties)).';</script>';

}


function _hasCurl(){
    return function_exists('curl_version');
}


function curl_file_get_contents($file)
{
	if (function_exists('curl_version'))
	{
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $file);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);

        $content = curl_exec($curl);
        curl_close($curl);
	}
	else if (file_get_contents(__FILE__) && ini_get('allow_url_fopen'))
	{
        $arrContextOptions=array(
            "ssl"=>array(
                "verify_peer"=>false,
                "verify_peer_name"=>false,
            ),
        ); 
        $content = file_get_contents($file, false, stream_context_create($arrContextOptions));
	}
	else
	{
        echo 'You have neither cUrl installed nor allow_url_fopen activated. Please setup one of those!';
	}
    return $content;
}

tuidd_render_login_button();
?>
